import axios from 'axios';
import qs from 'qs';
import { Modal } from 'antd';

const CancelToken = axios.CancelToken;
let cancel = {  };
export default class Api {
    _base;
    post( key, url, data, err ){
        return this._base.post(url, qs.stringify(data), {
             cancelToken: new CancelToken(function executor(c) {
            // An executor function receives a cancel function as a parameter
                cancel[key] = c;
            })
        }).catch(error => {
            if (axios.isCancel(error)) {
                return void 0;
            };
            let message = '系统错误';
            if (error.response) {
                message = error.response.data.message;
            } else if (error.request) {
                message = error.request;
            } else {
                message = error.message
            };
            if( err ){
                return err( message );
            };
            Modal.error({
                title: '系统提示',
                content: message
            });
        });
    };
    get( key, url, data, err ){
        return this._base.get(url, { 
            params: data,
            cancelToken: new CancelToken(function executor(c) {
            // An executor function receives a cancel function as a parameter
                cancel[key] = c;
            }) 
        }).catch(error => {
            if (axios.isCancel(error)) {
                return void 0;
            };
            let message = '系统错误';
            if (error.response) {
                message = error.response.data.message;
            } else if (error.request) {
                message = error.request;
            } else {
                message = error.message
            };
            if( err ){
                return err( message );
            };
            Modal.error({
                title: '系统提示',
                content: message
            });
        });
    };

    getCancel(name){
        return cancel[name];
    };

    all(...request){
        return axios.all(request);
    };

    constructor({ baseUrl='', apis={}}){
        this._axios = axios;

        this._base = axios.create({
            baseURL: baseUrl
        });

        for( let key in apis ){
            if( apis[key].mode === 'get' ){
                this[key] = ( data = {}, error) => {
                    return this.get( key, apis[key].url, data, error )
                };
            }else{
                this[key] = ( data = {}, error ) => {
                    return this.post( key, apis[key].url, data, error )
                };
            };    
        };
    };
};